/**
 *
 * This file is part of Tulip (www.tulip-software.org)
 *
 * Authors: David Auber, Patrick Mary, Morgan Mathiaut
 * from the LaBRI Visualization Team
 * Last modification : 15/01/2010
 *
 * Tulip is free software; you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Tulip is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 */
#ifndef Tulip_GLLODCALCULATOR_H
#define Tulip_GLLODCALCULATOR_H
#ifndef DOXYGEN_NOTFOR_DEVEL

#include <tulip/tulipconf.h>

#include <tulip/Matrix.h>
#include <tulip/Vector.h>

namespace tlp {

  class Camera;
  class GlEntity;
  class GlScene;
  class GlGraphInputData;

  enum RenderingEntitiesFlag {
    RenderingSimpleEntities=1,
    RenderingNodes=2,
    RenderingEdges=4,
    RenderingAll=7,
    RenderingWithoutRemove=8
  };

  typedef std::pair<unsigned long, float> LODResultSimpleEntity;
  typedef std::pair<unsigned int, float> LODResultComplexEntity;
  typedef std::vector<LODResultSimpleEntity> SimpleLODResultVector;
  typedef std::vector<LODResultComplexEntity> ComplexLODResultVector;
  typedef std::vector<SimpleLODResultVector> VectorOfSimpleLODResultVector;
  typedef std::vector<ComplexLODResultVector> VectorOfComplexLODResultVector;
  typedef std::vector<unsigned long> VectorOfCamera;

  /**
   * Class use to calculate lod of scene entities
   */
  class TLP_GL_SCOPE GlLODCalculator {

  public:

    virtual ~GlLODCalculator() {}
    virtual GlLODCalculator *clone()=0;

    /**
     * Set scene use by this LOD calculator
     */
    virtual void setScene(GlScene *scene){}

    /**
     * Set input data use to render
     */
    virtual void setInputData(GlGraphInputData *inputData) {}

    /**
     * Return if the LODCalculator need to have entities to compute
     */
    virtual bool needEntities() {return true;}
    /**
     * Set if the LODCalculator need to have entities to compute
     */
    virtual void setNeedEntities(bool) {}
    /**
     * Begin a new camera
     */
    virtual void beginNewCamera(Camera* camera)=0;
    /**
     * Record a new simple entity in current camera context
     */
    virtual void addSimpleEntityBoundingBox(unsigned long entity,const BoundingBox& bb)=0;
    /**
     * Record a new node in current camera context
     */
    virtual void addNodeBoundingBox(unsigned int id,const BoundingBox& bb)=0;
    /**
     * Record a new edge in current camera context
     */
    virtual void addEdgeBoundingBox(unsigned int id,const BoundingBox& bb)=0;

    /**
     * Compute all lod
     */
    virtual void compute(const Vector<int,4>& globalViewport,const Vector<int,4>& currentViewport,RenderingEntitiesFlag type)=0;

    /**
     * Return lod result for simple entities
     */
    virtual VectorOfSimpleLODResultVector* getResultForSimpleEntities() = 0;
    /**
     * Return lod result for nodes
     */
    virtual VectorOfComplexLODResultVector* getResultForNodes() = 0;
    /**
     * Return lod result for edges
     */
    virtual VectorOfComplexLODResultVector* getResultForEdges() = 0;
    /**
     * Return cameras vector
     */
    virtual VectorOfCamera* getVectorOfCamera() = 0;
    /**
     * Clear class data
     */
    virtual void clear() = 0;
  };

}

#endif // DOXYGEN_NOTFOR_DEVEL

#endif // Tulip_GLLODCALCULATOR_H
